@@ -20,11 +20,16 @@
  * Boston, MA  02110-1301  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "gedit-view-frame.h"
 #include "gedit-marshal.h"
 #include "gedit-debug.h"
 #include "gedit-utils.h"
-#include "gedit-overlay.h"
+#include "gedit-animated-overlay.h"
+#include "gedit-floating-slider.h"
 #include "gedit-rounded-frame.h"
 
 #include <gdk/gdkkeysyms.h>
@@ -66,12 +71,14 @@ struct _GeditViewFramePrivate
 	 */
 	guint        search_flags;
 
+	GtkWidget   *slider;
 	GtkWidget   *search_widget;
 	GtkWidget   *search_entry;
 
 	guint        typeselect_flush_timeout;
 	glong        view_scroll_event_id;
 	glong        search_entry_focus_out_id;
+	glong        search_entry_changed_id;
 
 	guint        disable_popdown : 1;
 	guint        wrap_around : 1;
@@ -154,6 +161,9 @@ hide_search_widget (GeditViewFrame *frame,
 		return;
 	}
 
+	g_signal_handler_block (frame->priv->search_entry,
+	                        frame->priv->search_entry_focus_out_id);
+
 	if (frame->priv->view_scroll_event_id != 0)
 	{
 		g_signal_handler_disconnect (frame->priv->view,
@@ -161,21 +171,16 @@ hide_search_widget (GeditViewFrame *frame,
 		frame->priv->view_scroll_event_id = 0;
 	}
 
-	if (frame->priv->search_entry_focus_out_id != 0)
-	{
-		g_signal_handler_disconnect (frame->priv->search_entry,
-		                             frame->priv->search_entry_focus_out_id);
-		frame->priv->search_entry_focus_out_id = 0;
-	}
-
 	if (frame->priv->typeselect_flush_timeout != 0)
 	{
 		g_source_remove (frame->priv->typeselect_flush_timeout);
 		frame->priv->typeselect_flush_timeout = 0;
 	}
 
-	gtk_widget_destroy (frame->priv->search_widget);
-	frame->priv->search_widget = NULL;
+	/* To hide the slider we just set the animation-state property */
+	g_object_set (G_OBJECT (frame->priv->slider),
+	              "animation-state", GEDIT_THEATRICS_ANIMATION_STATE_INTENDING_TO_GO,
+	              NULL);
 
 	if (cancel)
 	{
@@ -196,6 +201,9 @@ hide_search_widget (GeditViewFrame *frame,
 	/* Make sure the view is the one who has the focus when we destroy
 	   the search widget */
 	gtk_widget_grab_focus (frame->priv->view);
+
+	g_signal_handler_unblock (frame->priv->search_entry,
+	                          frame->priv->search_entry_focus_out_id);
 }
 
 static void
@@ -278,44 +286,37 @@ static void
 set_entry_background (GtkWidget               *entry,
                       GeditSearchEntryBgColor  col)
 {
+	GdkRGBA *fg, *bg;
+
 	if (col == GEDIT_SEARCH_ENTRY_NOT_FOUND)
 	{
-		GdkColor error_default_bg = { 0, 0xff00, 0x6600, 0x6600 };
-		GdkColor error_default_fg = { 0, 0xff00, 0xff00, 0xff00 };
-		GdkColor sym_bg, sym_fg;
-		GdkColor fg, bg;
-		GtkStyle *style;
+		GdkRGBA error_default_fg = { 0.65, 0.15, 0.15, 1.0 };
+		GdkRGBA error_default_bg = { 0.93, 0.21, 0.21, 1.0 };
+		GdkRGBA sym_bg, sym_fg;
+		GtkStyleContext *context;
 
-		style = gtk_widget_get_style (entry);
+		context = gtk_widget_get_style_context (entry);
 
-		if (gtk_style_lookup_color (style, "error_fg_color", &sym_fg) &&
-		    gtk_style_lookup_color (style, "error_bg_color", &sym_bg))
+		if (gtk_style_context_lookup_color (context, "error_fg_color", &sym_fg) &&
+		    gtk_style_context_lookup_color (context, "error_bg_color", &sym_bg))
 		{
-			fg = sym_fg;
-			bg = sym_bg;
+			fg = &sym_fg;
+			bg = &sym_bg;
 		}
 		else
 		{
-			fg = error_default_fg;
-			bg = error_default_bg;
+			fg = &error_default_fg;
+			bg = &error_default_bg;
 		}
-
-		gtk_widget_modify_base (entry,
-		                        GTK_STATE_NORMAL,
-		                        &bg);
-		gtk_widget_modify_text (entry,
-		                        GTK_STATE_NORMAL,
-		                        &fg);
 	}
 	else /* reset */
 	{
-		gtk_widget_modify_base (entry,
-		                        GTK_STATE_NORMAL,
-		                        NULL);
-		gtk_widget_modify_text (entry,
-		                        GTK_STATE_NORMAL,
-		                        NULL);
+		fg = NULL;
+		bg = NULL;
 	}
+
+	gtk_widget_override_background_color (entry, 0, bg);
+	gtk_widget_override_color (entry, 0, fg);
 }
 
 static gboolean
@@ -481,12 +482,14 @@ search_widget_scroll_event (GtkWidget      *widget,
 		return retval;
 
 	/* SEARCH mode */
-	if (event->direction == GDK_SCROLL_UP)
+	if ((event->state & GDK_CONTROL_MASK) &&
+	    event->direction == GDK_SCROLL_UP)
 	{
 		search_again (frame, TRUE);
 		retval = TRUE;
 	}
-	else if (event->direction == GDK_SCROLL_DOWN)
+	else if ((event->state & GDK_CONTROL_MASK) &&
+	         event->direction == GDK_SCROLL_DOWN)
 	{
 		search_again (frame, FALSE);
 		retval = TRUE;
@@ -566,10 +569,39 @@ search_widget_key_press_event (GtkWidget      *widget,
 	return retval;
 }
 
+static void
+update_search (GeditViewFrame *frame)
+{
+	if (frame->priv->search_mode == SEARCH)
+	{
+		GeditDocument *doc;
+		const gchar *entry_text;
+		gchar *search_text;
+		guint  search_flags;
+
+		doc = gedit_view_frame_get_document (frame);
+
+		entry_text = gtk_entry_get_text (GTK_ENTRY (frame->priv->search_entry));
+
+		search_text = gedit_document_get_search_text (doc, &search_flags);
+
+		if ((search_text == NULL) ||
+		    (strcmp (search_text, entry_text) != 0) ||
+		     search_flags != frame->priv->search_flags)
+		{
+			gedit_document_set_search_text (doc,
+			                                entry_text,
+			                                frame->priv->search_flags);
+		}
+
+		g_free (search_text);
+	}
+}
+
 static void
 wrap_around_menu_item_toggled (GtkCheckMenuItem *checkmenuitem,
                                GeditViewFrame   *frame)
-{	
+{
 	frame->priv->wrap_around = gtk_check_menu_item_get_active (checkmenuitem);
 }
 
@@ -579,6 +611,7 @@ match_entire_word_menu_item_toggled (GtkCheckMenuItem *checkmenuitem,
 {
 	GEDIT_SEARCH_SET_ENTIRE_WORD (frame->priv->search_flags,
 	                              gtk_check_menu_item_get_active (checkmenuitem));
+	update_search (frame);
 }
 
 static void
@@ -587,6 +620,7 @@ match_case_menu_item_toggled (GtkCheckMenuItem *checkmenuitem,
 {
 	GEDIT_SEARCH_SET_CASE_SENSITIVE (frame->priv->search_flags,
 	                                 gtk_check_menu_item_get_active (checkmenuitem));
+	update_search (frame);
 }
 
 static void
@@ -892,7 +926,6 @@ static void
 search_init (GtkWidget      *entry,
              GeditViewFrame *frame)
 {
-	GeditDocument *doc;
 	const gchar *entry_text;
 
 	/* renew the flush timeout */
@@ -905,27 +938,11 @@ search_init (GtkWidget      *entry,
 			               frame);
 	}
 
-	doc = gedit_view_frame_get_document (frame);
-
 	entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
 
 	if (frame->priv->search_mode == SEARCH)
 	{
-		gchar *search_text;
-		guint  search_flags;
-
-		search_text = gedit_document_get_search_text (doc, &search_flags);
-
-		if ((search_text == NULL) ||
-		    (strcmp (search_text, entry_text) != 0) ||
-		     search_flags != frame->priv->search_flags)
-		{
-			gedit_document_set_search_text (doc,
-			                                entry_text,
-			                                frame->priv->search_flags);
-		}
-
-		g_free (search_text);
+		update_search (frame);
 
 		run_search (frame,
 		            entry_text,
@@ -944,6 +961,9 @@ search_init (GtkWidget      *entry,
 			gchar **split_text = NULL;
 			const gchar *text;
 			GtkTextIter iter;
+			GeditDocument *doc;
+
+			doc = gedit_view_frame_get_document (frame);
 
 			gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),
 			                                  &iter,
@@ -1067,14 +1087,10 @@ create_search_widget (GeditViewFrame *frame)
 	g_signal_connect (search_widget, "scroll-event",
 	                  G_CALLBACK (search_widget_scroll_event),
 	                  frame);
-	/* Manage the scroll also for the view */
-	frame->priv->view_scroll_event_id =
-		g_signal_connect (frame->priv->view, "scroll-event",
-			          G_CALLBACK (search_widget_scroll_event),
-			          frame);
 
 	hbox = gtk_hbox_new (FALSE, 3);
 	gtk_widget_show (hbox);
+
 	gtk_container_add (GTK_CONTAINER (search_widget), hbox);
 	gtk_container_set_border_width (GTK_CONTAINER (hbox), 3);
 
@@ -1096,9 +1112,10 @@ create_search_widget (GeditViewFrame *frame)
 	g_signal_connect (frame->priv->search_entry, "insert_text",
 	                  G_CALLBACK (search_entry_insert_text),
 	                  frame);
-	g_signal_connect (frame->priv->search_entry, "changed",
-	                  G_CALLBACK (search_init),
-	                  frame);
+	frame->priv->search_entry_changed_id =
+		g_signal_connect (frame->priv->search_entry, "changed",
+		                  G_CALLBACK (search_init),
+		                  frame);
 	frame->priv->search_entry_focus_out_id =
 		g_signal_connect (frame->priv->search_entry, "focus-out-event",
 		                  G_CALLBACK (search_entry_focus_out_event),
@@ -1194,6 +1211,8 @@ init_search_entry (GeditViewFrame *frame)
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (frame->priv->view));
 
+	customize_for_search_mode (frame);
+
 	if (frame->priv->search_mode == GOTO_LINE)
 	{
 		gint   line;
@@ -1210,6 +1229,8 @@ init_search_entry (GeditViewFrame *frame)
 
 		gtk_entry_set_text (GTK_ENTRY (frame->priv->search_entry),
 		                    line_str);
+		gtk_editable_select_region (GTK_EDITABLE (frame->priv->search_entry),
+		                            0, -1);
 
 		g_free (line_str);
 
@@ -1228,11 +1249,6 @@ init_search_entry (GeditViewFrame *frame)
 
 		old_find_text = gedit_document_get_search_text (GEDIT_DOCUMENT (buffer),
 		                                                &old_find_flags);
-		if (old_find_text != NULL)
-		{
-			frame->priv->old_search_text = old_find_text;
-			add_search_completion_entry (old_find_text);
-		}
 
 		if (old_find_flags != 0)
 		{
@@ -1247,28 +1263,37 @@ init_search_entry (GeditViewFrame *frame)
 		{
 			gtk_entry_set_text (GTK_ENTRY (frame->priv->search_entry),
 			                    find_text);
+			gtk_editable_set_position (GTK_EDITABLE (frame->priv->search_entry),
+			                           -1);
 		}
-		else
+		else if (old_find_text != NULL)
 		{
+			frame->priv->old_search_text = old_find_text;
+			add_search_completion_entry (old_find_text);
+			g_signal_handler_block (frame->priv->search_entry,
+			                        frame->priv->search_entry_changed_id);
+
 			gtk_entry_set_text (GTK_ENTRY (frame->priv->search_entry),
-			                    "");
+			                    old_find_text);
+			gtk_editable_select_region (GTK_EDITABLE (frame->priv->search_entry),
+			                            0, -1);
+
+			g_signal_handler_unblock (frame->priv->search_entry,
+			                          frame->priv->search_entry_changed_id);
 		}
 
 		g_free (find_text);
 	}
 }
 
-static gboolean
+static void
 start_interactive_search_real (GeditViewFrame *frame)
 {
 	GtkTextBuffer *buffer;
-	GtkTextMark *mark;
 	GtkTextIter iter;
+	GtkTextMark *mark;
 
-	/* FIXME: it enters here twice, why? */
-
-	if ((frame->priv->search_widget != NULL) &&
-	    gtk_widget_get_visible (frame->priv->search_widget))
+	if (gtk_widget_get_visible (frame->priv->slider))
 	{
 		if (frame->priv->search_mode != frame->priv->request_search_mode)
 		{
@@ -1276,7 +1301,9 @@ start_interactive_search_real (GeditViewFrame *frame)
 		}
 		else
 		{
-			return TRUE;
+			gtk_editable_select_region (GTK_EDITABLE (frame->priv->search_entry),
+			                            0, -1);
+			return;
 		}
 	}
 
@@ -1286,39 +1313,65 @@ start_interactive_search_real (GeditViewFrame *frame)
 
 	if (frame->priv->search_mode == SEARCH)
 	{
-		mark = gtk_text_buffer_get_selection_bound (buffer);
+		GtkTextIter start, end;
+
+		if (gtk_text_buffer_get_selection_bounds (buffer, &start, &end))
+		{
+			if (gtk_text_iter_compare (&start, &end) == -1)
+			{
+				iter = start;
+			}
+			else
+			{
+				iter = end;
+			}
+		}
+		else
+		{
+			mark = gtk_text_buffer_get_selection_bound (buffer);
+			gtk_text_buffer_get_iter_at_mark (buffer, &iter, mark);
+		}
 	}
 	else
 	{
 		mark = gtk_text_buffer_get_insert (buffer);
+		gtk_text_buffer_get_iter_at_mark (buffer, &iter, mark);
 	}
 
-	gtk_text_buffer_get_iter_at_mark (buffer, &iter, mark);
 	frame->priv->start_mark = gtk_text_buffer_create_mark (buffer, NULL,
 	                                                       &iter, FALSE);
 
-	frame->priv->search_widget = create_search_widget (frame);
+	/* To slide in we set the right animation state in the animatable */
+	g_object_set (G_OBJECT (frame->priv->slider),
+	              "animation-state", GEDIT_THEATRICS_ANIMATION_STATE_COMING,
+	              NULL);
 
-	gedit_overlay_add_animated_widget (GEDIT_OVERLAY (frame->priv->overlay),
-	                                   frame->priv->search_widget,
-	                                   300,
-	                                   GEDIT_THEATRICS_CHOREOGRAPHER_EASING_EXPONENTIAL_IN_OUT,
-	                                   GEDIT_THEATRICS_CHOREOGRAPHER_BLOCKING_DOWNSTAGE,
-	                                   GTK_ORIENTATION_VERTICAL,
-	                                   GDK_GRAVITY_SOUTH_WEST,
-	                                   SEARCH_POPUP_OFFSET);
+	/* NOTE: we must be very careful here to not have any text before
+	   focusing the entry because when the entry is focused the text is
+	   selected, and gtk+ doesn't allow us to have more than one selection
+	   active */
+	g_signal_handler_block (frame->priv->search_entry,
+	                        frame->priv->search_entry_changed_id);
+	gtk_entry_set_text (GTK_ENTRY (frame->priv->search_entry),
+	                    "");
+	g_signal_handler_unblock (frame->priv->search_entry,
+	                          frame->priv->search_entry_changed_id);
+
+	/* We need to grab the focus after the widget has been added */
+	gtk_widget_grab_focus (frame->priv->search_entry);
 
 	init_search_entry (frame);
 
+	/* Manage the scroll also for the view */
+	frame->priv->view_scroll_event_id =
+		g_signal_connect (frame->priv->view, "scroll-event",
+			          G_CALLBACK (search_widget_scroll_event),
+			          frame);
+
 	frame->priv->typeselect_flush_timeout =
 		g_timeout_add (GEDIT_VIEW_FRAME_SEARCH_DIALOG_TIMEOUT,
 		               (GSourceFunc) search_entry_flush_timeout,
 		               frame);
-
-	/* We need to grab the focus after the widget has been added */
-	gtk_widget_grab_focus (frame->priv->search_entry);
-
-	return TRUE;
 }
 
 static void
@@ -1361,29 +1414,14 @@ view_frame_mount_operation_factory (GeditDocument   *doc,
 	return gtk_mount_operation_new (GTK_WINDOW (window));
 }
 
-static gboolean
-on_start_interactive_search (GeditView      *view,
-                             GeditViewFrame *frame)
-{
-	frame->priv->request_search_mode = SEARCH;
-
-	return start_interactive_search_real (frame);
-}
-
-static gboolean
-on_start_interactive_goto_line (GeditView      *view,
-                                GeditViewFrame *frame)
-{
-	frame->priv->request_search_mode = GOTO_LINE;
-
-	return start_interactive_search_real (frame);
-}
-
 static void
 gedit_view_frame_init (GeditViewFrame *frame)
 {
 	GeditDocument *doc;
 	GtkWidget *sw;
+	GeditOverlayChildPosition position;
+	GtkStyleContext *context;
+	GdkRGBA bg_color;
 
 	frame->priv = GEDIT_VIEW_FRAME_GET_PRIVATE (frame);
 
@@ -1401,33 +1439,58 @@ gedit_view_frame_init (GeditViewFrame *frame)
 
 	g_object_unref (doc);
 
-	g_signal_connect (frame->priv->view,
-	                  "start-interactive-search",
-	                  G_CALLBACK (on_start_interactive_search),
-	                  frame);
-
-	g_signal_connect (frame->priv->view,
-	                  "start-interactive-goto-line",
-	                  G_CALLBACK (on_start_interactive_goto_line),
-	                  frame);
-
-	frame->priv->overlay = gedit_overlay_new (frame->priv->view);
-	gtk_widget_show (frame->priv->overlay);
-
 	/* Create the scrolled window */
 	sw = gtk_scrolled_window_new (NULL, NULL);
 
-	gtk_container_add (GTK_CONTAINER (sw), frame->priv->overlay);
+	gtk_container_add (GTK_CONTAINER (sw), frame->priv->view);
 
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
 
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
-					     GTK_SHADOW_IN);
 	gtk_widget_show (sw);
 
-	gtk_box_pack_start (GTK_BOX (frame), sw, TRUE, TRUE, 0);
+	frame->priv->overlay = gedit_animated_overlay_new (sw, frame->priv->view);
+
+	gedit_overlay_set_composited (GEDIT_OVERLAY (frame->priv->overlay), TRUE);
+	gtk_widget_show (frame->priv->overlay);
+
+	gtk_box_pack_start (GTK_BOX (frame), frame->priv->overlay, TRUE, TRUE, 0);
+
+	/* Add slider */
+	if (gtk_widget_get_direction (GTK_WIDGET (frame)) == GTK_TEXT_DIR_RTL)
+	{
+		position = GEDIT_OVERLAY_CHILD_POSITION_NORTH_WEST;
+	}
+	else
+	{
+		position = GEDIT_OVERLAY_CHILD_POSITION_NORTH_EAST;
+	}
+
+	frame->priv->search_widget = create_search_widget (frame);
+	frame->priv->slider = gedit_floating_slider_new (frame->priv->search_widget);
+	g_object_set (G_OBJECT (frame->priv->slider),
+	              "position", position,
+	              "offset", SEARCH_POPUP_OFFSET,
+	              "easing", GEDIT_THEATRICS_CHOREOGRAPHER_EASING_EXPONENTIAL_IN_OUT,
+	              "blocking", GEDIT_THEATRICS_CHOREOGRAPHER_BLOCKING_DOWNSTAGE,
+	              "orientation", GTK_ORIENTATION_VERTICAL,
+	              NULL);
+
+	/* Force search widget slider to be transparent */
+	context = gtk_widget_get_style_context (frame->priv->slider);
+	gtk_style_context_get_background_color (context,
+	                                        GTK_STATE_NORMAL,
+	                                        &bg_color);
+
+	bg_color.alpha = 0;
+
+	gtk_widget_override_background_color (frame->priv->slider,
+	                                      GTK_STATE_NORMAL,
+	                                      &bg_color);
+
+	gedit_animated_overlay_add (GEDIT_ANIMATED_OVERLAY (frame->priv->overlay),
+	                            GEDIT_ANIMATABLE (frame->priv->slider));
 }
 
 GeditViewFrame *
@@ -1451,3 +1514,42 @@ gedit_view_frame_get_view (GeditViewFrame *frame)
 
 	return GEDIT_VIEW (frame->priv->view);
 }
+
+void
+gedit_view_frame_popup_search (GeditViewFrame *frame)
+{
+	g_return_if_fail (GEDIT_IS_VIEW_FRAME (frame));
+
+	frame->priv->request_search_mode = SEARCH;
+
+	start_interactive_search_real (frame);
+}
+
+void
+gedit_view_frame_popup_goto_line (GeditViewFrame *frame)
+{
+	g_return_if_fail (GEDIT_IS_VIEW_FRAME (frame));
+
+	frame->priv->request_search_mode = GOTO_LINE;
+
+	start_interactive_search_real (frame);
+}
+
+void
+gedit_view_frame_clear_search (GeditViewFrame *frame)
+{
+	GeditDocument *doc;
+
+	g_return_if_fail (GEDIT_IS_VIEW_FRAME (frame));
+
+	doc = gedit_view_frame_get_document (frame);
+
+	gedit_document_set_search_text (doc, "", GEDIT_SEARCH_DONT_SET_FLAGS);
+	g_signal_handler_block (frame->priv->search_entry,
+	                        frame->priv->search_entry_changed_id);
+	gtk_entry_set_text (GTK_ENTRY (frame->priv->search_entry), "");
+	g_signal_handler_unblock (frame->priv->search_entry,
+	                          frame->priv->search_entry_changed_id);
+
+	gtk_widget_grab_focus (frame->priv->view);
+}
